package com.mcapraro.ld28
{
	import starling.text.TextFieldAutoSize;
	import starling.text.TextField;
	import flash.events.KeyboardEvent;
	import starling.events.TouchPhase;
	import starling.events.TouchEvent;
	import starling.events.Event;
	import starling.display.Image;
	import starling.display.Sprite;
	import com.greensock.TweenMax;
	import Box2D.Dynamics.Contacts.b2PolyAndCircleContact;
	import citrus.physics.box2d.Box2DUtils;
	import citrus.core.CitrusObject;
	import citrus.view.starlingview.StarlingArt;
	import starling.display.BlendMode;
	import citrus.objects.platformer.box2d.Coin;
	import citrus.objects.platformer.box2d.Sensor;
	import starling.display.Quad;
	import starling.display.Shape;
	import citrus.objects.CitrusSprite;
	import citrus.utils.objectmakers.ObjectMaker2D;
	import flash.display.MovieClip;
	import citrus.physics.box2d.Box2D;
	import org.osflash.signals.Signal;
	import citrus.core.starling.StarlingState;

	/**
	 * @author Michelangelo Capraro (m&#64;mcapraro.com)
	 */
	public class Level1State extends StarlingState
	{
		public var onRestart:Signal;
		private var box2D:Box2D;
		private var _level:MovieClip;
		private var _hero:BirdHero;
		private var _nest:Sensor;
		private var talkBubble:TalkBubble;
		private var inventoryUI:InventoryUI;
		private var includes:Array;
		private var _female:Sensor;
		private var skyLayerSprite:CitrusSprite;
		private var bgTiledTrees:TiledBackground;
		private var bgTiledTrees2:TiledBackground;
		private var backLayerSprite:CitrusSprite;
		private var middleLayerSprite:CitrusSprite;
		private var frontLayerSprite:CitrusObject;
		private var _firstRun:Boolean;
		private var dialog:Sprite;
		
		public function Level1State(firstRun:Boolean = false)
		{
			trace("Level1State created");
			super();
			
			includes = [ItemPlaceholder];
			
			_level = new Level1();
			
			onRestart = new Signal(int);
			
			_firstRun = firstRun;
		}
		
		override public function initialize():void
		{
			super.initialize();
			
			box2D = new Box2D("box2D");
			//box2D.visible = true;
			add(box2D);
			
			
			setupMap();
			setupHero();
			setupObjects();
			setupCamera();
			setupUI();		
						
			start();

		}

		private function setupUI():void
		{
			inventoryUI = new InventoryUI(12);
			addChild(inventoryUI);
			
		}

		private function start():void
		{
			talkBubble = new TalkBubble();
			talkBubble.x = (view.getArt(_female) as StarlingArt).getBounds(this).x;
			talkBubble.y = (view.getArt(_female) as StarlingArt).getBounds(this).y - _female.height / 2;
			talkBubble.onHidden.add(onTalkBubbleHidden);
			addChild(talkBubble);
			
			if (_firstRun) {
				//TODO: make this a class, make it more memroy efficient, etc
				dialog = new Sprite();
				dialog.addChild(new Image(Assets.assets.getTexture("splash_screen")));
				dialog.x = (stage.stageWidth - dialog.width) / 2;
				dialog.y = (stage.stageHeight - dialog.height) / 2;
				dialog.alpha = 0;
				dialog.addEventListener(TouchEvent.TOUCH, onShowInstructions);
				addChild(dialog);
				
				_ce.stage.addEventListener(KeyboardEvent.KEY_DOWN, onKeyShowInstructions);	
				
				TweenMax.to(dialog, .5, {alpha:1});
			} else {
				talkBubble.show();
			}

		}
		
		private function onShowInstructions(event:TouchEvent):void
		{
			if (event.touches[0].phase == TouchPhase.ENDED) {
				showInstructions();
			}
		}

		private function onKeyShowInstructions(event:KeyboardEvent):void
		{
			showInstructions();
		}

		private function showInstructions():void
		{
			dialog.addChild(new Image(Assets.assets.getTexture("instructions_screen")));
			
			dialog.removeEventListeners();
			_ce.stage.removeEventListener(KeyboardEvent.KEY_DOWN, onKeyShowInstructions);	
			
			dialog.addEventListener(TouchEvent.TOUCH, onDismissSplash);
			_ce.stage.addEventListener(KeyboardEvent.KEY_DOWN, onKeyDismissSplash);	
		}
		
		private function onDismissSplash(event:TouchEvent):void
		{
			if (event.touches[0].phase == TouchPhase.ENDED) {
				hideSplashScreen();
			}
		}
		
		private function onKeyDismissSplash(event:KeyboardEvent):void
		{
			hideSplashScreen();
		}
		
		private function hideSplashScreen():void
		{
			TweenMax.to(dialog, .5, {alpha:0, onComplete:function():void{
				_ce.stage.removeEventListener(KeyboardEvent.KEY_DOWN, onKeyDismissSplash);	
				dialog.removeEventListeners();
				removeChild(dialog);
				talkBubble.show();
			}});
		}
		
		private function onTalkBubbleHidden():void
		{
			talkBubble.destroy();
			removeChild(talkBubble, true);
			releaseHero();
		}
		
		private function releaseHero():void 
		{
			_hero.inverted = false;

			camera.target = _hero;
			_hero.controlsEnabled = true;
			TweenMax.delayedCall(1.5, function():void { camera.easing.x = 0.25; });
		}

		private function setupCamera():void
		{
			//camera.bounds = new Rectangle(0,0,2000,2000);
			camera.target = _nest;
			camera.enabled = true;
			camera.easing.x = 0.05;//0.25;
			//camera.allowZoom = true;
			//camera.setZoom(.2);
			camera.reset();
						
		}

		private function setupHero():void
		{
			_hero = getObjectByName("hero") as BirdHero; // call this to be sure we have a reference to the hero set
			_hero.group = 2; // set htis on top of the background (at group 0)
			_hero.controlsEnabled = false;
			_hero.hurtDuration = 200;
			_hero.inverted = true;
		}

		private function setupObjects():void
		{
			_nest = getObjectByName("nest") as Sensor;
			_nest.group = 1;
			_nest.view = NestAsset;
			_nest.onBeginContact.add(onReturnToNest);
			
			_female = getObjectByName("female") as Sensor;
			_female.group = 1;
			_female.view = FemaleBirdAsset;
			
			// create items to pick up, double the amount required and add a few
			var tempItems:Vector.<Class> = (_ce.gameData as GameData).requiredItems.concat();
			for (var i:int = 0; i < (_ce.gameData as GameData).requiredItemCount + 4; i++) {
				tempItems.push((_ce.gameData as GameData).items[Math.ceil(Math.random() * ((_ce.gameData as GameData).items.length - 1))]);
			}

			// now find the ItemPlaceholder Sensors on the stage and randomle replace them with some of the items.
			var placeHolders:Vector.<CitrusObject> = getObjectsByType(ItemPlaceholder);
			Utils.shuffleVector(placeHolders as Vector.<*>);
			
			// now loop through the new aray and create coins
			var itemCoin:Coin;
			for (i = 0; i < Math.min(tempItems.length, placeHolders.length); i++) {
				itemCoin = new Coin("item_" + i, {group:2, view:tempItems[i], x:(placeHolders[i] as ItemPlaceholder).x, y:(placeHolders[i] as ItemPlaceholder).y});
				itemCoin.onBeginContact.add(onItemGrabbed);
				add(itemCoin);
			}
						
		}

		private function onReturnToNest(callback:b2PolyAndCircleContact):void
		{
			callback;
			var validated:Boolean = (_ce.gameData as GameData).validateCollectedItems();
			
			_hero.controlsEnabled = false;
			talkBubble = new TalkBubble();
			talkBubble.x = Math.abs((view.getArt(_female) as StarlingArt).getBounds((view.getArt(_female) as StarlingArt).parent).x);
			talkBubble.y = (view.getArt(_female) as StarlingArt).getBounds((view.getArt(_female) as StarlingArt).parent).y - _female.height + 10;
								
			addChild(talkBubble);
			talkBubble.show(true, validated);
			
			TweenMax.delayedCall(1, function():void{showEndScreen(validated);});
			
		}

		private function showEndScreen(validated:Boolean):void
		{
			dialog = new Sprite();
			dialog.addChild(new Image(Assets.assets.getTexture(validated ? "round_screen" :"game_over_screen")));
			dialog.x = (stage.stageWidth - dialog.width) / 2;
			dialog.y = (stage.stageHeight - dialog.height) / 2;
			dialog.alpha = 0;
			
			
			if (validated) {
				var textField:TextField = new TextField(100, 20, "round " + ((_ce.gameData as GameData).requiredItemCount + 1), "MuseoSlab-500", 28, 0xffffff);	
				textField.x = 183;
				textField.y = 132;
				textField.autoSize = TextFieldAutoSize.BOTH_DIRECTIONS;		
				dialog.addChild(textField);	
			}
			
			TweenMax.delayedCall(2, function():void{
				dialog.addEventListener(TouchEvent.TOUCH, validated ? onDismissDialog : onDismissEndDialog);
				_ce.stage.addEventListener(KeyboardEvent.KEY_DOWN, validated ? onKeyDismissDialog : onKeyDismissEndDialog);	
			});

			addChild(dialog);
			
			TweenMax.to(dialog, .5, {alpha:1});
		}

		private function onKeyDismissDialog(event:KeyboardEvent):void
		{
			dismissDialog();
		}

		private function onKeyDismissEndDialog(event:KeyboardEvent):void
		{
			dismissEndDialog();
		}

		private function onDismissEndDialog(event:TouchEvent):void
		{
			if (event.touches[0].phase == TouchPhase.ENDED) {
				dismissEndDialog();
			}
		}

		private function dismissEndDialog():void
		{
			TweenMax.to(dialog, .5, {alpha:0, onComplete:function():void{
				_ce.stage.removeEventListener(KeyboardEvent.KEY_DOWN, onKeyDismissDialog);	
				_ce.stage.removeEventListener(KeyboardEvent.KEY_DOWN, onKeyDismissEndDialog);	
				dialog.removeEventListeners();
				removeChild(dialog);
				onRestart.dispatch((_ce.gameData as GameData).requiredItemCount);
			}});
		}

		private function onDismissDialog(event:TouchEvent):void
		{
			if (event.touches[0].phase == TouchPhase.ENDED) {
				dismissDialog();
			}
		}

		private function dismissDialog():void
		{
			TweenMax.to(dialog, .5, {alpha:0, onComplete:function():void{
				_ce.stage.removeEventListener(KeyboardEvent.KEY_DOWN, onKeyDismissDialog);	
				_ce.stage.removeEventListener(KeyboardEvent.KEY_DOWN, onKeyDismissEndDialog);	
				dialog.removeEventListeners();
				removeChild(dialog);
				onRestart.dispatch((_ce.gameData as GameData).requiredItemCount + 1);
			}});
		}

		private function onItemGrabbed(callback:b2PolyAndCircleContact):void
		{		
			_hero.hurt();
			inventoryUI.addItem((Box2DUtils.CollisionGetObjectByType(ItemPlaceholder, callback) as Coin).view);
			(_ce.gameData as GameData).collectItem((Box2DUtils.CollisionGetObjectByType(ItemPlaceholder, callback) as Coin).view);
		}

		private function setupMap():void
		{
			// create the physics bodies from the SWF
			ObjectMaker2D.FromMovieClip(_level);

			// create background
			var topColor:uint    = 0x39A1BF;
			var bottomColor:uint = 0xF6C2DF;
			var skyQuad:Quad = new Quad(stage.stageWidth, stage.stageHeight);
			skyQuad.setVertexColor(0, topColor);
			skyQuad.setVertexColor(1, topColor);
			skyQuad.setVertexColor(2, bottomColor);
			skyQuad.setVertexColor(3, bottomColor);
			skyQuad.blendMode = BlendMode.NONE;
			
			skyLayerSprite = new CitrusSprite("skyLayer", { x:0, y:0, group:0, view:skyQuad, parallaxX:0, parallaxY:0, touchable:false } );
			add(skyLayerSprite);
						
			// add the tiling trees
			bgTiledTrees = new TiledBackground("TiledTrees", {x:0, y:0, offsetX: -2000, width:4000, height:640, group:0, texture:"trees", parallaxX:.1, parallaxY:0, alpha:.6});
			add(bgTiledTrees);

			bgTiledTrees2 = new TiledBackground("TiledTrees2", {x:0, y:0, offsetX:-2500, width:4000, height:640, group:0, texture:"trees", parallaxX:.2, parallaxY:0});
			add(bgTiledTrees2);

			// create geometry from the SWF file shape data
			var backLayer:Shape = new Shape();
			ShapeConverter.convertToGeometry(_level.getChildByName("back"), backLayer);
			backLayerSprite = new CitrusSprite("backLayer", { x:0, y:0, group:0, view:backLayer, parallaxX:.5, touchable:false } );
			add(backLayerSprite);

			// this uses the geometry-based map
			var middleLayer:Shape = new Shape();
			ShapeConverter.convertToGeometry(_level.getChildByName("middle"), middleLayer);
			middleLayerSprite = new CitrusSprite("frontLayer", { x:0, y:0, group:1, view:middleLayer, parallaxX:1, touchable:false } );
			add(middleLayerSprite);

			// this uses the geometry-based map
			var frontLayer:Shape = new Shape();
			ShapeConverter.convertToGeometry(_level.getChildByName("front"), frontLayer);
			frontLayerSprite = new CitrusSprite("frontLayer", { x:0, y:0, group:3, view:frontLayer, parallaxX:1.2, touchable:false } );
			add(frontLayerSprite);

		}
		
	
		override public function update(timeDelta:Number):void
		{
			super.update(timeDelta);
		}


		override public function destroy():void
		{
			talkBubble.destroy();
			removeChild(talkBubble, true);
			super.destroy();
		}
		
	}
}
