package com.mcapraro.ld28
{
	import starling.textures.Texture;
	import starling.core.Starling;
	import flash.geom.Matrix;
	import flash.display.BitmapData;
	import flash.display.Sprite;
	import citrus.core.CitrusEngine;
	import com.greensock.TweenMax;
	import starling.display.Image;
	import org.osflash.signals.Signal;
	import starling.display.Shape;
	import starling.events.Event;

	/**
	 * @author Michelangelo Capraro (m&#64;mcapraro.com)
	 */
	public class TalkBubble extends starling.display.Sprite
	{
		private var rectangle:Shape;
		private var triangle:Shape;
		private var items:Vector.<Class>;
		public var onHidden:Signal;
		private var itemContainer:starling.display.Sprite;
		
		public function TalkBubble()
		{
			onHidden = new Signal();
			items = new <Class>[CoinBerryAsset, CoinButtonAsset, CoinCapAsset, CoinLeafAsset, CoinMoneyAsset, CoinRockAsset, CoinShellAsset, CoinStringAsset, CoinTwigAsset, CoinHeartAsset];
			itemContainer = new starling.display.Sprite();
			alpha = 0;
			addEventListener(Event.ADDED_TO_STAGE, onAdded);
		}

		private function onAdded(event:Event):void
		{
			event;
				
			// draw the pointed bit
			triangle = new Shape();
			triangle.graphics.beginFill(0x221F20);
			triangle.graphics.moveTo(-20, -20);
			triangle.graphics.lineTo(20, -20);
			triangle.graphics.lineTo(0, 0);
			triangle.graphics.lineTo(-20, -20);
			triangle.graphics.endFill();
			
			addChild(triangle);
			
			
			// draw the rectangle
			rectangle = new Shape();
			rectangle.graphics.beginFill(0x221F20);
			rectangle.graphics.drawRect(0, 0, 206, 92);
			rectangle.graphics.endFill();
			addChild(rectangle);
			
			addChild(itemContainer);
			
		}

		private function update():void
		{
			
			rectangle.width = itemContainer.width + 16;
			rectangle.height = itemContainer.height + 16;
			rectangle.x = -rectangle.width / 2;
			rectangle.y = - (triangle.height + rectangle.height);
			
			if (getBounds(parent).x < 20) {
				rectangle.x += Math.abs(getBounds(parent).x) + 20;
			}			
			
			itemContainer.x = rectangle.x + 8;
			itemContainer.y = rectangle.y + 8;
		}
		
		public function destroy():void
		{	
			onHidden.removeAll();
			removeChildren(0, -1, true);
			triangle = null;
			rectangle = null;
			items = null;
		}
		
		/**
		 * this number passed in here is the number of items the bubble will shoe to the player
		 */
		public function show(validateMode:Boolean = false, validation:Boolean = false):void
		{
			var itemClass:Class;
			var itemCount:int = (CitrusEngine.getInstance().gameData as GameData).requiredItemCount;
			
			if (validation) {
				var itemMC:flash.display.Sprite = new CoinHeartAsset();
				var itemBMD:BitmapData = new BitmapData(64, 68, true, 0x00000000);
				itemBMD.draw(itemMC, new Matrix(1, 0, 0, 1, 32, 34));

				itemContainer.addChild(new Image(Texture.fromBitmapData(itemBMD, false, false, Starling.contentScaleFactor)));
			} else {
				
				for (var i:int = 0; i < itemCount; i++) {
					
					itemClass = (CitrusEngine.getInstance().gameData as GameData).requiredItems[i];
									
					var itemImage:Image = itemContainer.addChild(new Image((CitrusEngine.getInstance().gameData as GameData).itemTextures[itemClass])) as Image;
					itemImage.x = (i % 12) * 64;	
					itemImage.y = Math.floor(i / 12) * 68;	
				}
			}
			
			update();
			
			TweenMax.to(this, 1, {delay:.3, alpha:1});
			if (!validateMode) TweenMax.to(this, 1, {delay:1 + itemCount, alpha:0, onComplete:onHidden.dispatch});
			
		}
	}
}
